#pragma NOIV							// Do not generate interrupt vectors

#include "fx2.h"
#include "fx2regs.h"
#include "syncdly.h"					// SYNCDELAY macro

#define DIP3_in	bmBIT0
#define DCLK_out	bmBIT1
#define DATA0_out	bmBIT3
#define DONE_in	bmBIT7

#define DIP3	PA0					// vierter DIP-Schalter (1->CPLD, 0->8051)
#define DCLK	PA1					// Datenclock
#define DATA0	PA3					// Datenleitung
#define DONE	PA7					// CONFIG_DONE

#define setConfIO() OEA = DCLK_out | DATA0_out	// set some Port A pins as outputs
#define triConfIO() OEA = 0x00				// tristates Port A I/Os

extern BOOL		GotSUD;							// Received setup data flag
extern BOOL		Sleep;
extern BOOL		Rwuen;
extern BOOL		Selfpwr;

BOOL cfg_failed;

BYTE				Configuration;					// Current configuration
BYTE				AlternateSetting;				// Alternate settings

//**********************************************************************************
// Table of registers which require a synchronization delay:
//
// ECC1B0			ECC1B1				ECC1B2			ECC2B0				ECC2B1
// ECC2B2			ECCCFG				ECCRESET			EPxAUTOINLENH:L 	EPxBCH:L 
// EPxFIFOCFG 		EPxFIFOIE			EPxFIFOIRQ		EPxFIFOPFH:L		EPxGPIFFLGSEL 
//	EPxGPIFTRIG		FIFOPINPOLAR		FIFORESE			GPIFADRH:L 			GPIFIE 
// GPIFIRQ			GPIFTCB0				GPIFTCB1			GPIFTCB2				GPIFTCB3
// GPIFTRIG			INPKTEND 			OUTPKTEND		PINFLAGSAB			PINFLAGSCD
// REVCTL			UDMACRCH:L

//**********************************************************************************
//* Task Dispatcher hooks
//* The following hooks are called by the task dispatcher.
//**********************************************************************************

void TD_Init(void)								// Called once at startup
{

	CPUCS = 0x10;									// set the CPU clock to 48MHz
   SYNCDELAY;

	//-------------------------------------------------------------------------------
	// configure I/O ports
	PINFLAGSAB = 0x00;							// FLAGA = PF, FLAGB = FF, FLAGC = EF
	SYNCDELAY;
	PINFLAGSCD = 0x00;
	SYNCDELAY;

	PORTACFG = 0x00;								// I/O-functionality of PORTA pins
	WAKEUPCS = 0xC5;								// disable wakeup functionality of WU2EN pin

	triConfIO();									// tristate FPGA configuration outputs
	DCLK = 0;
	DATA0 = 0;

	//-------------------------------------------------------------------------------
	// configure endpoints, uncommented register assignment => register reset value
	//
	IFCONFIG = 0x03;								// ext. IFCLK, inv. IFCLK, synch. slave FIFO
	SYNCDELAY;

	REVCTL = 0x03;									// REVCTL.0 and REVCTL.1 to set to 1
	
	EP1OUTCFG = 0xA0;								// IN, Bulk, 64 Byte
	SYNCDELAY;
	EP1INCFG = 0xA0;								// OUT, Bulk, 64 Byte
	SYNCDELAY;

	EP2CFG = 0xA2;									// OUT, Bulk, 512 Byte, double buffered
	SYNCDELAY;
	EP4CFG = 0xA0;									// OUT, Bulk, 512 Byte, double buffered
	SYNCDELAY;
	EP6CFG = 0xE2;									// IN, Bulk, 512 Byte, double buffered
	SYNCDELAY;
	EP8CFG = 0xE0; 								// IN, Bulk, 512 Byte, double buffered
	SYNCDELAY;

	FIFOPINPOLAR = 0x00;							// polarity of FIFO control inputs
	SYNCDELAY;
	
	FIFORESET = 0x80;								// reset all FIFOs
	SYNCDELAY;
	FIFORESET = 0x02;								// reset EP2 
	SYNCDELAY;
	FIFORESET = 0x04;								// reset EP4 
	SYNCDELAY;
	FIFORESET = 0x06;								// reset EP6
	SYNCDELAY;
	FIFORESET = 0x08;								// reset EP8
	SYNCDELAY;
	FIFORESET = 0x00; 
	SYNCDELAY; 

	// since the defaults are double buffered we must write dummy byte counts twice
	OUTPKTEND = 0x82;								// Arm both EP2 buffers to prime the pump
	SYNCDELAY;
	OUTPKTEND = 0x82;
	SYNCDELAY;
	OUTPKTEND = 0x84;								// Arm both EP4 buffers to prime the pump
	SYNCDELAY;
	OUTPKTEND = 0x84;
	SYNCDELAY;

	EP2FIFOCFG = 0x11;							// AUTOOUT, WORDWIDE
	SYNCDELAY; 
	EP4FIFOCFG = 0x11;							// AUTOOUT, WORDWIDE
	SYNCDELAY; 
	EP6FIFOCFG = 0x09;							// OUTOIN, WORDWIDE
	SYNCDELAY; 
	EP8FIFOCFG = 0x09;							// OUTOIN, WORDWIDE
	SYNCDELAY; 

	EP6AUTOINLENH = 0x02;						// 512 Bytes
	SYNCDELAY; 
	EP6AUTOINLENL = 0x00;
	SYNCDELAY;
	EP8AUTOINLENH = 0x02;						// 512 Bytes
	SYNCDELAY; 
	EP8AUTOINLENL = 0x00;
	SYNCDELAY;

	// out endpoints do not come up armed
	EP1OUTBC = 0x40;								// arm EP1OUT by writing byte count w/skip.
	SYNCDELAY;

//	EPxFIFOPFH/L									// programmable level flag (unused)
// EPxFIFOIE
// EP2FIFOIRQ

	//-------------------------------------------------------------------------------
	// interrupt configuration
	EIE = 0x00;										// disable INT4 & ext. ints

	EPIE = 0x08;									// EP1 OUT Interrupt Enable
	EPIE = 0x08;									// EP1 OUT Interrupt Enable

	AUTOPTRSETUP |= 0x01;						// enable dual autopointer feature
	Rwuen = TRUE;									// Enable remote-wakeup

	cfg_failed = FALSE;
}

//**********************************************************************************
//
void TD_Poll(void)								// Called repeatedly while the device is idle
{
	if(DIP3 == 0 && DONE == 0 && cfg_failed == TRUE)
	{
		cfg_failed = FALSE;
		EP1OUTBC = 0x40;								// arm EP1OUT by writing byte count w/skip.
	}

	if(DONE==1 || DIP3 == 1)
	{
		triConfIO();
	}
}

BOOL TD_Suspend(void)          // Called before the device goes into suspend mode
{
   return(TRUE);
}

BOOL TD_Resume(void)          // Called after the device resumes
{
   return(TRUE);
}


//-----------------------------------------------------------------------------
// Device Request hooks
//   The following hooks are called by the end point 0 device request parser.
//-----------------------------------------------------------------------------

BOOL DR_GetDescriptor(void)
{
   return(TRUE);
}

BOOL DR_SetConfiguration(void)   // Called when a Set Configuration command is received
{
   Configuration = SETUPDAT[2];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetConfiguration(void)   // Called when a Get Configuration command is received
{
   EP0BUF[0] = Configuration;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_SetInterface(void)       // Called when a Set Interface command is received
{
   AlternateSetting = SETUPDAT[2];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetInterface(void)       // Called when a Set Interface command is received
{
   EP0BUF[0] = AlternateSetting;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_GetStatus(void)
{
   return(TRUE);
}

BOOL DR_ClearFeature(void)
{
   return(TRUE);
}

BOOL DR_SetFeature(void)
{
   return(TRUE);
}

BOOL DR_VendorCmnd(void)
{
   return(TRUE);
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//   The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------

// Setup Data Available Interrupt Handler
void ISR_Sudav(void) interrupt 0
{
   GotSUD = TRUE;            // Set flag
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUDAV;         // Clear SUDAV IRQ
}

// Setup Token Interrupt Handler
void ISR_Sutok(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUTOK;         // Clear SUTOK IRQ
}

void ISR_Sof(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSOF;            // Clear SOF IRQ
}

void ISR_Ures(void) interrupt 0
{
   // whenever we get a USB reset, we should revert to full speed mode
   pConfigDscr = pFullSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
   pOtherConfigDscr = pHighSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;
   
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmURES;         // Clear URES IRQ
}

void ISR_Susp(void) interrupt 0
{
   Sleep = TRUE;
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUSP;
}

void ISR_Highspeed(void) interrupt 0
{
   if (EZUSB_HIGHSPEED())
   {
      pConfigDscr = pHighSpeedConfigDscr;
      ((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
      pOtherConfigDscr = pFullSpeedConfigDscr;
      ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;
   }

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmHSGRANT;
}
void ISR_Ep0ack(void) interrupt 0
{
}
void ISR_Stub(void) interrupt 0
{
}
void ISR_Ep0in(void) interrupt 0
{
}
void ISR_Ep0out(void) interrupt 0
{
}
void ISR_Ep1in(void) interrupt 0
{
/*	EP1OUTBC = 0x40;								// arm EP1OUT by writing byte count w/skip.
	SYNCDELAY;

	EZUSB_IRQ_CLEAR();
	EPIRQ = bmBIT2;*/
}

// simply sends received data back to USB host
void ISR_Ep1out(void) interrupt 0
{
	BYTE i, j, shift_reg, count;

	count = EP1OUTBC;

	if(DIP3 == 0 && DONE == 0 )				// FPGA nur dann konfigurieren, wenn dieser unkonfiguriert ist und CPLD abgeschaltet ist
	{
		setConfIO();
		for(i=0; i<count ; i++)
		{
			// Daten LSB first rausschieben
			shift_reg = EP1OUTBUF[i];
			for(j=0; j<8; j++)
			{
				DCLK = 0;
				if(shift_reg & 0x01) { DATA0 = 1; }
				else { DATA0 = 0; }
				DCLK = 1;
				shift_reg = shift_reg >> 1;
			}
		}	

		EP1OUTBC = 0x40;								// arm EP1OUT by writing byte count w/skip.
		SYNCDELAY;
	}
	else
	{
		cfg_failed = TRUE;
		triConfIO();									// tristate FPGA configuration outputs
	}

	EZUSB_IRQ_CLEAR();
	EPIRQ = bmBIT3;
}
void ISR_Ep2inout(void) interrupt 0
{
}
void ISR_Ep4inout(void) interrupt 0
{
}
void ISR_Ep6inout(void) interrupt 0
{
}
void ISR_Ep8inout(void) interrupt 0
{
}
void ISR_Ibn(void) interrupt 0
{
}
void ISR_Ep0pingnak(void) interrupt 0
{
}
void ISR_Ep1pingnak(void) interrupt 0
{
}
void ISR_Ep2pingnak(void) interrupt 0
{
}
void ISR_Ep4pingnak(void) interrupt 0
{
}
void ISR_Ep6pingnak(void) interrupt 0
{
}
void ISR_Ep8pingnak(void) interrupt 0
{
}
void ISR_Errorlimit(void) interrupt 0
{
}
void ISR_Ep2piderror(void) interrupt 0
{
}
void ISR_Ep4piderror(void) interrupt 0
{
}
void ISR_Ep6piderror(void) interrupt 0
{
}
void ISR_Ep8piderror(void) interrupt 0
{
}
void ISR_Ep2pflag(void) interrupt 0
{
}
void ISR_Ep4pflag(void) interrupt 0
{
}
void ISR_Ep6pflag(void) interrupt 0
{
}
void ISR_Ep8pflag(void) interrupt 0
{
}
void ISR_Ep2eflag(void) interrupt 0
{
}
void ISR_Ep4eflag(void) interrupt 0
{
}
void ISR_Ep6eflag(void) interrupt 0
{
}
void ISR_Ep8eflag(void) interrupt 0
{
}
void ISR_Ep2fflag(void) interrupt 0
{
}
void ISR_Ep4fflag(void) interrupt 0
{
}
void ISR_Ep6fflag(void) interrupt 0
{
}
void ISR_Ep8fflag(void) interrupt 0
{
}
void ISR_GpifComplete(void) interrupt 0
{
}
void ISR_GpifWaveform(void) interrupt 0
{
}
